program Lights;

uses
	Windows, Messages, MMSystem, Math,
	Direct3D9, D3DX9;

////////////////////////////////////////////////////////////////////////////////

type
	T3DBase = class
	public
		procedure Process(); virtual; abstract;
	end;

////////////////////////////////////////////////////////////////////////////////

type
	T3DCamera = class(T3DBase)
	private
		m_pD3DDevice : IDirect3DDevice9;
		m_modified   : boolean;
		m_vecEye     : TD3DVector;
		m_vecLookAt  : TD3DVector;
		m_vecUp      : TD3DVector;
		m_angle      : single;
		m_height     : single;
		m_radius     : single;

		procedure m_SetAngle (angle : single);
		procedure m_SetHeight(height: single);
		procedure m_SetRadius(radius: single);
		procedure m_Apply();

	public
		constructor Create(const pD3DDevice: IDirect3DDevice9);

		procedure Process(); override;

		property Angle  : single read m_angle  write m_SetAngle;
		property Height : single read m_height write m_SetHeight;
		property Radius : single read m_radius write m_Setradius;

	end;

procedure T3DCamera.m_SetAngle(angle: single);
begin
	m_angle := angle;
	m_modified := TRUE;
end;

procedure T3DCamera.m_SetHeight(height: single);
begin
	m_height := height;
	m_modified := TRUE;
end;

procedure T3DCamera.m_SetRadius(radius: single);
begin
	m_radius := radius;
	m_modified := TRUE;
end;

procedure T3DCamera.m_Apply();
var
	matView: TD3DMatrix;
begin
	m_vecEye := D3DXVector3(m_radius * cos(m_angle), m_height, m_radius * sin(m_angle));
	D3DXMatrixLookAtLH(matView, m_vecEye, m_vecLookAt, m_vecUp);

	m_pD3DDevice.SetTransform(D3DTS_VIEW, matView);
end;

constructor T3DCamera.Create(const pD3DDevice: IDirect3DDevice9);
begin
	m_pD3DDevice := pD3DDevice;

	m_modified  := FALSE;

	m_vecLookAt := D3DXVector3Zero;
	m_vecUp     := D3DXVector3(0.0, 1.0, 0.0);

	m_angle  := 0.0;
	m_height := 3.0;//12.0;
	m_radius := -7.5;//-10.0;

	m_Apply();
end;

procedure T3DCamera.Process();
begin
	if m_modified then begin
		m_Apply();
		m_modified := FALSE;
	end;
end;

////////////////////////////////////////////////////////////////////////////////

const
	LOW_INDEX  = 0;
	HIGH_INDEX = 100;

type
	TProjectionMode = (pmPerspecive, pmOrthogonal);

	T3DProjection = class(T3DBase)
	private
		m_pD3DDevice: IDirect3DDevice9;
		m_projectionMatrix: array[LOW_INDEX..HIGH_INDEX] of TD3DMatrix;
		m_projectionMode: TProjectionMode;
		m_projectionStep: integer;

		procedure m_InitiaizeProjectionMatrix();
		procedure m_SetProjectionMode(mode: TProjectionMode);
		procedure m_Apply();

	public
		constructor Create(const pD3DDevice: IDirect3DDevice9);
		procedure Process(); override;

		property  Mode: TProjectionMode read m_projectionMode write m_SetProjectionMode;
	end;

procedure T3DProjection.m_InitiaizeProjectionMatrix();
var
	matProj1, matProj2: TD3DMatrix;
	factor: single;
	i, j, k: integer;
begin
	D3DXMatrixPerspectiveFovLH(matProj1, D3DX_PI/4, 4/3, 1.0, 100.0);
	D3DXMatrixOrthoOffCenterLH(matProj2, -6.0, 6.0, -6.0*3/4, 6.0*3/4, 1.0, 100.0);

	for k := LOW_INDEX to HIGH_INDEX do begin
		factor := k * 1.0 + 0.1;
		factor := (log2(1.0 / factor) - 0.0);
		for j := 0 to 3 do begin
			for i := 0 to 3 do begin
				m_projectionMatrix[k].m[i,j] := matProj1.m[i,j] + (matProj2.m[i,j] - matProj1.m[i,j]) * factor / (log2(1.0/HIGH_INDEX)-0.0);
			end;
		end;
	end;

	m_projectionMatrix[LOW_INDEX]  := matProj1;
	m_projectionMatrix[HIGH_INDEX] := matProj2;
end;

procedure T3DProjection.m_SetProjectionMode(mode: TProjectionMode);
begin
	if m_projectionMode = mode then
		exit;

	inc(m_projectionStep);
end;

procedure T3DProjection.m_Apply();
var
	matProject: TD3DMatrix;
begin
	if m_projectionStep > 0 then begin
		if m_projectionMode = pmPerspecive then begin
			matProject := m_projectionMatrix[m_projectionStep];
		end else begin
			matProject := m_projectionMatrix[HIGH_INDEX-m_projectionStep];
		end;

		inc(m_projectionStep);
		if m_projectionStep >= HIGH_INDEX then begin
			m_projectionStep := LOW_INDEX;
			if m_projectionMode = pmPerspecive then
				m_projectionMode := pmOrthogonal
			else
				m_projectionMode := pmPerspecive
		end;
	end
	else begin
		if m_projectionMode = pmPerspecive then
			matProject := m_projectionMatrix[LOW_INDEX]
		else
			matProject := m_projectionMatrix[HIGH_INDEX];
	end;

	m_pD3DDevice.SetTransform(D3DTS_PROJECTION, matProject);
end;

constructor T3DProjection.Create(const pD3DDevice: IDirect3DDevice9);
begin
	m_pD3DDevice := pD3DDevice;

	m_projectionStep := LOW_INDEX;

	m_InitiaizeProjectionMatrix();
	m_Apply();
end;

procedure T3DProjection.Process();
begin
	if m_projectionStep > 0 then
		m_Apply();
end;

////////////////////////////////////////////////////////////////////////////////

type
	T3DLight = class(T3DBase)
	private
		m_pD3DDevice: IDirect3DDevice9;
		m_light: TD3DLight9;

		procedure m_Apply();

	public
		constructor Create(const pD3DDevice: IDirect3DDevice9);
		procedure Process(); override;

	end;

constructor T3DLight.Create(const pD3DDevice: IDirect3DDevice9);
var
	vecDirection: TD3DXVector3;
begin
	m_pD3DDevice := pD3DDevice;

	ZeroMemory(@m_light, sizeof(m_light));

	m_light._Type     := D3DLIGHT_DIRECTIONAL;
	m_light.Diffuse.r := 0.6;
	m_light.Diffuse.g := 0.6;
	m_light.Diffuse.b := 0.6;
	m_light.Range     := 1000.0;

	vecDirection      := D3DXVector3(-5.0, -5.0, 5.0);
	D3DXVec3Normalize(m_light.Direction, vecDirection);

	m_pD3DDevice.LightEnable(0, TRUE);
	m_pD3DDevice.SetRenderState(D3DRS_LIGHTING, 1);

	m_pD3DDevice.SetRenderState(D3DRS_AMBIENT, $00606060);

	m_Apply();
end;

procedure T3DLight.Process();
begin
end;

procedure T3DLight.m_Apply();
begin
	m_pD3DDevice.SetLight(0, m_light);
end;

////////////////////////////////////////////////////////////////////////////////

const
	FOG_START : single = 8.0*1.0;
	FOG_END   : single = 15.0*1.0;

type
	T3DFog = class(T3DBase)
	private
		m_pD3DDevice: IDirect3DDevice9;
		m_fogStart: single;
		m_fogEnd: single;

		procedure m_Apply();

	public
		constructor Create(const pD3DDevice: IDirect3DDevice9);
		procedure Process(); override;

	end;

constructor T3DFog.Create(const pD3DDevice: IDirect3DDevice9);
begin
	m_pD3DDevice := pD3DDevice;

	m_pD3DDevice.SetRenderState(D3DRS_FOGCOLOR, $00000000);
	m_pD3DDevice.SetRenderState(D3DRS_FOGENABLE, 1);

	m_pD3DDevice.SetRenderState(D3DRS_FOGVERTEXMODE, D3DFOG_LINEAR);
	m_pD3DDevice.SetRenderState(D3DRS_FOGSTART, Plongword(@FOG_START)^);
	m_pD3DDevice.SetRenderState(D3DRS_FOGEND, Plongword(@FOG_END)^);

	m_pD3DDevice.SetRenderState(D3DRS_RANGEFOGENABLE, 1);

	m_Apply();
end;

procedure T3DFog.Process();
begin
end;

procedure T3DFog.m_Apply();
begin
end;

////////////////////////////////////////////////////////////////////////////////

type
	T3DAttrib = class(T3DBase)
	private
		m_pD3DDevice: IDirect3DDevice9;
		m_camera: T3DCamera;
		m_projection: T3DProjection;
		m_light: T3DLight;
		m_fog: T3DFog;

	public
		constructor Create(const pD3DDevice: IDirect3DDevice9);
		procedure Free();
		procedure Process(); override;

		property Camera: T3DCamera read m_camera write m_camera;
		property Projection: T3DProjection read m_projection write m_projection;

	end;

constructor T3DAttrib.Create(const pD3DDevice: IDirect3DDevice9);
begin
	inherited Create();

	// assign Direct3D device instance
	m_pD3DDevice := pD3DDevice;

	// initialize internal instances
	m_camera := T3DCamera.Create(pD3DDevice);
	m_projection := T3DProjection.Create(pD3DDevice);
	m_light := T3DLight.Create(pD3DDevice);
	m_fog := T3DFog.Create(pD3DDevice);
end;

procedure T3DAttrib.Free();
begin
	m_fog.Free();
	m_light.Free();
	m_projection.Free();
	m_camera.Free();
end;

procedure T3DAttrib.Process();
begin
	m_camera.Process();
	m_projection.Process();
end;

var
	g_3DAttrib: T3DAttrib;

////////////////////////////////////////////////////////////////////////////////

var
	g_pD3D: IDirect3D9 = nil;
	g_pd3dDevice: IDirect3DDevice9 = nil;
	g_pFont: ID3DXFont;

function InitD3D(hWnd: HWND): HRESULT;
var
	d3dpp: TD3DPresentParameters;
	mtrl: TD3DMaterial9;
begin
	Result := E_FAIL;

	g_pD3D := Direct3DCreate9(D3D_SDK_VERSION);
	if (g_pD3D = nil) then
		Exit;

	FillChar(d3dpp, SizeOf(d3dpp), 0);
	d3dpp.Windowed := True;
	d3dpp.SwapEffect := D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat := D3DFMT_UNKNOWN;
	d3dpp.EnableAutoDepthStencil := True;
	d3dpp.AutoDepthStencilFormat := D3DFMT_D16;
{
	d3dpp.Windowed := FALSE;
	d3dpp.BackBufferCount := 1;
	d3dpp.BackBufferFormat := D3DFMT_A8R8G8B8;
	d3dpp.BackBufferWidth := 1024;
	d3dpp.BackBufferHeight := 768;
}
	// Create the D3DDevice
	Result:= g_pD3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
							   D3DCREATE_SOFTWARE_VERTEXPROCESSING,
							   @d3dpp, g_pd3dDevice);
	if FAILED(Result) then begin
		Result:= E_FAIL;
		Exit;
	end;

//	g_pd3dDevice.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	g_pd3dDevice.SetRenderState(D3DRS_ZENABLE, iTrue);

	g_pd3dDevice.SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	g_pd3dDevice.SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

	g_pD3dDevice.SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
	g_pD3dDevice.SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);

//	g_pD3dDevice.SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	g_pD3DDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	g_pD3DDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	g_pD3DDevice.SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);

	// setup material
	ZeroMemory(@mtrl, SizeOf(TD3DMaterial9));
	mtrl.Diffuse.r := 1.0; mtrl.Ambient.r := 1.0;
	mtrl.Diffuse.g := 1.0; mtrl.Ambient.g := 1.0;
	mtrl.Diffuse.b := 1.0; mtrl.Ambient.b := 1.0;
	mtrl.Diffuse.a := 1.0; mtrl.Ambient.a := 1.0;
	g_pd3dDevice.SetMaterial(mtrl);

	// create font
	D3DXCreateFont(g_pd3dDevice, 24, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET,
				 OUT_DEFAULT_PRECIS, ANTIALIASED_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
				 'Courier New', g_pFont);

	// setup attribute
	g_3DAttrib := T3DAttrib.Create(g_pd3dDevice);

	Result:= S_OK;
end;

procedure Cleanup;
begin
	g_3DAttrib.Free();

	if (g_pd3dDevice <> nil) then
		g_pd3dDevice:= nil;

	if (g_pD3D <> nil) then
		g_pD3D:= nil;
end;

procedure SetupMatrices;
var
	matWorld: TD3DMatrix;
begin
	D3DXMatrixIdentity(matWorld);

//	D3DXMatrixRotationY(matWorld, s_angle - PI / 360.0);
	g_pd3dDevice.SetTransform(D3DTS_WORLD, matWorld);
end;

//////////////////////////////////

type
	TFPoint = record
		x, y: Single;
	end;

	TFRect = record
		x1, y1, x2, y2: Single;
	end;

	function FPoint(x, y: single): TFPoint;
	begin
		result.x := x;
		result.y := y;
	end;


type
	TD3DTextureCoord = TFPoint;

	PSmVertex = ^TSmVertex;
	TSmVertex = packed record
		position: TD3DVector;
		normal: TD3DVector;
		texPos: TD3DTextureCoord;
	end;

	PSmVertexArray = ^TSmVertexArray;
	TSmVertexArray = array [0..0] of TSmVertex;

	PSmVertexTL = ^TSmVertexTL;
	TSmVertexTL = packed record
		position: TD3DVector;
		rhw: single;
		color: longword;
		texPos: TD3DTextureCoord;
	end;

const
	D3DFVF_SMVERTEX = (D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1);

function MakeVertexXZ(x1, z1, x2, z2: single; y1: single; out vertexArray: TSmVertex): integer;
const
	ynormal = -1.0;
var
	i, index: integer;
	vertices: PSmVertexArray;
begin
	vertices := PSmVertexArray(addr(vertexArray));

	index := 0;
	vertices[index].position := D3DXVector3(x1, y1, z1); inc(index);
	vertices[index].position := D3DXVector3(x2, y1, z1); inc(index);
	vertices[index].position := D3DXVector3(x1, y1, z2); inc(index);
	vertices[index].position := D3DXVector3(x1, y1, z2); inc(index);
	vertices[index].position := D3DXVector3(x2, y1, z1); inc(index);
	vertices[index].position := D3DXVector3(x2, y1, z2); inc(index);

	for i := 0 to pred(index) do
		vertices[i].normal := D3DXVector3(0.0, ynormal, 0.0);

	result := index;
end;

function MakeVertexXYZ(x1, y1, z1, x2, y2, z2: single; tile: integer; out vertexArray: TSmVertex): integer;
type
	TCubeVertex = (cv111, cv121, cv221, cv211, cv112, cv122, cv222, cv212);
	TTexturePos = (tp00, tp10, tp01, tp11);
var
	index: integer;
	cube: array[TCubeVertex] of TD3DVector;
	texPos: array[TTexturePos] of TD3DTextureCoord;
	normal: TD3DVector;
	vertices: PSmVertexArray;
begin
	cube[cv111] := D3DXVector3(x1, y1, z1);
	cube[cv121] := D3DXVector3(x1, y2, z1);
	cube[cv221] := D3DXVector3(x2, y2, z1);
	cube[cv211] := D3DXVector3(x2, y1, z1);
	cube[cv112] := D3DXVector3(x1, y1, z2);
	cube[cv122] := D3DXVector3(x1, y2, z2);
	cube[cv222] := D3DXVector3(x2, y2, z2);
	cube[cv212] := D3DXVector3(x2, y1, z2);

	texPos[tp00].x := (tile*36.0+2.0) / 512.0;
	texPos[tp00].y := 2.0 / 64.0;
	texPos[tp10].x := texPos[tp00].x + 32.0 / 512.0;
	texPos[tp10].y := texPos[tp00].y;
	texPos[tp01].x := texPos[tp00].x;
	texPos[tp01].y := texPos[tp00].y + 32.0 / 64.0;
	texPos[tp11].x := texPos[tp10].x;
	texPos[tp11].y := texPos[tp01].y;

	vertices := PSmVertexArray(addr(vertexArray));

	index := 0;

	if (x1 = x2) then begin
		normal := D3DXVector3(1.0, 0.0, 0.0);
		D3DXVec3normalize(normal, normal);
		vertices[index].position := cube[cv121]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp00]; inc(index);
		vertices[index].position := cube[cv122]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp10]; inc(index);
		vertices[index].position := cube[cv111]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp01]; inc(index);
		vertices[index]          := vertices[index-1]; inc(index);
		vertices[index]          := vertices[index-3]; inc(index);
		vertices[index].position := cube[cv112]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp11]; inc(index);
	end
	else if (y1 = y2) then begin
		normal := D3DXVector3(0.0, 1.0, 0.0);
		D3DXVec3normalize(normal, normal);
		vertices[index].position := cube[cv112]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp00]; inc(index);
		vertices[index].position := cube[cv212]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp10]; inc(index);
		vertices[index].position := cube[cv111]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp01]; inc(index);
		vertices[index]          := vertices[index-1]; inc(index);
		vertices[index]          := vertices[index-3]; inc(index);
		vertices[index].position := cube[cv211]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp11]; inc(index);
	end
	else if (z1 = z2) then begin
		normal := D3DXVector3(0.0, 0.0, -1.0);
		D3DXVec3normalize(normal, normal);
		vertices[index].position := cube[cv121]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp00]; inc(index);
		vertices[index].position := cube[cv221]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp10]; inc(index);
		vertices[index].position := cube[cv111]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp01]; inc(index);
		vertices[index]          := vertices[index-1]; inc(index);
		vertices[index]          := vertices[index-3]; inc(index);
		vertices[index].position := cube[cv211]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp11]; inc(index);
	end
	else begin
		normal := D3DXVector3(0.0, 0.0, z1-z2);
		D3DXVec3normalize(normal, normal);
		vertices[index].position := cube[cv121]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp00]; inc(index);
		vertices[index].position := cube[cv221]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp10]; inc(index);
		vertices[index].position := cube[cv111]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp01]; inc(index);
		vertices[index]          := vertices[index-1]; inc(index);
		vertices[index]          := vertices[index-3]; inc(index);
		vertices[index].position := cube[cv211]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp11]; inc(index);

		normal := D3DXVector3(x1-x2, 0.0, 0.0);
		D3DXVec3normalize(normal, normal);
		vertices[index].position := cube[cv122]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp00]; inc(index);
		vertices[index].position := cube[cv121]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp10]; inc(index);
		vertices[index].position := cube[cv112]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp01]; inc(index);
		vertices[index]          := vertices[index-1]; inc(index);
		vertices[index]          := vertices[index-3]; inc(index);
		vertices[index].position := cube[cv111]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp11]; inc(index);

		normal := D3DXVector3(0.0, 0.0, z2-z1);
		D3DXVec3normalize(normal, normal);
		vertices[index].position := cube[cv222]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp00]; inc(index);
		vertices[index].position := cube[cv122]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp10]; inc(index);
		vertices[index].position := cube[cv212]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp01]; inc(index);
		vertices[index]          := vertices[index-1]; inc(index);
		vertices[index]          := vertices[index-3]; inc(index);
		vertices[index].position := cube[cv112]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp11]; inc(index);

		normal := D3DXVector3(x2-x1, 0.0, 0.0);
		D3DXVec3normalize(normal, normal);
		vertices[index].position := cube[cv221]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp00]; inc(index);
		vertices[index].position := cube[cv222]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp10]; inc(index);
		vertices[index].position := cube[cv211]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp01]; inc(index);
		vertices[index]          := vertices[index-1]; inc(index);
		vertices[index]          := vertices[index-3]; inc(index);
		vertices[index].position := cube[cv212]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp11]; inc(index);

		normal := D3DXVector3(0.0, y2-y1, 0.0);
		D3DXVec3normalize(normal, normal);
		vertices[index].position := cube[cv122]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp00]; inc(index);
		vertices[index].position := cube[cv222]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp10]; inc(index);
		vertices[index].position := cube[cv121]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp01]; inc(index);
		vertices[index]          := vertices[index-1]; inc(index);
		vertices[index]          := vertices[index-3]; inc(index);
		vertices[index].position := cube[cv221]; vertices[index].normal := normal; vertices[index].texPos := texPos[tp11]; inc(index);
	end;

	result := index;
end;

function Map2Vertives(const map: array of longword; mapTPL: integer; out vertices: array of TSmVertex): integer;
var
	x, z: integer;
	numVertices: integer;
	wLoop, hLoop: integer;
	wMap, hMap, zMap: integer;
	tile, height: integer;
begin
	wMap := mapTPL;
	hMap := succ(high(map)) div wMap;

	numVertices := 0;

	for hLoop := 0 to pred(hMap) do
	for wLoop := 0 to pred(wMap) do begin
		x := wLoop - wMap div 2;
		z := hLoop - hMap div 2;
		tile := map[hLoop*mapTPL+wLoop] and $FFFF;
		height := map[hLoop*mapTPL+wLoop] shr 16;

		if (height = 0) then begin
			inc(numVertices, MakeVertexXYZ(-0.5+x*1.0, 0.0, -0.5+z*1.0, 0.5+x*1.0, 0.0, 0.5+z*1.0, tile, vertices[numVertices]));
		end
		else begin
			for zMap := 1 to height do begin
				inc(numVertices, MakeVertexXYZ(-0.5+x*1.0, (zMap-1)*1.0, -0.5+z*1.0, 0.5+x*1.0, zMap*1.0, 0.5+z*1.0, tile, vertices[numVertices]));
			end;
		end;

	end;

	result := numVertices;
end;

const
	S_MAP_WIDTH  = 16;
	S_MAP_HEIGHT = 16;

const
	s_pTile: IDirect3DTexture9 = nil;

var
	s_map: array[0..pred(S_MAP_WIDTH*S_MAP_HEIGHT)] of longword;
	s_numVertices: integer;
	s_vertices: array[0..S_MAP_WIDTH*S_MAP_HEIGHT*6*5*3] of TSmVertex;

procedure Test();
const
	s_isFirst: boolean = TRUE;
	s_pTexture: IDirect3DTexture9 = nil;
	s_pSprite: IDirect3DTexture9 = nil;
var
	x, y: integer;
	angle: single;
	vertices: array[0..3] of TSmVertex;
	fpPoint: array[0..2] of TFPoint;

begin
	if s_isFirst then begin
		D3DXCreateTextureFromFileA(g_pD3DDevice, './tile.bmp', s_pTexture);
		D3DXCreateTextureFromFileA(g_pD3DDevice, './chara.png', s_pSprite);
		D3DXCreateTextureFromFileA(g_pD3DDevice, './NewNeto1.bmp', s_pTile);

		Randomize();
		for y := 0 to pred(S_MAP_HEIGHT) do begin
			for x := 0 to pred(S_MAP_WIDTH) do begin
//				s_map[y*S_MAP_WIDTH + x] := (random(3) shl 16) or random(9);
				case random(10) of
					0..7: s_map[y*S_MAP_WIDTH + x] := ((0) shl 16) or 8;
					8: s_map[y*S_MAP_WIDTH + x] := ((1) shl 16) or 4;
					9: s_map[y*S_MAP_WIDTH + x] := ((0) shl 16) or 6;
				end;
			end;
		end;
		s_numVertices := Map2Vertives(s_map, S_MAP_WIDTH, s_vertices);
		s_isFirst := FALSE;
	end;

	g_pd3dDevice.SetTexture(0, s_pTexture);
	g_pd3dDevice.SetFVF(D3DFVF_SMVERTEX);
	g_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLELIST, s_numVertices div 3, s_vertices, sizeof(TSmVertex));

	g_pD3DDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
	begin
		angle := g_3DAttrib.Camera.Angle;
		vertices[0].position := D3DXVector3(-0.5 * sin(angle), 1.0, 0.5 * cos(angle));
		vertices[0].normal   := D3DXVector3(0.0, 0.0, -1.0);
		vertices[0].texPos.x := 0.0;
		vertices[0].texPos.y := 0.0;
		vertices[1].position := D3DXVector3(0.5 * sin(angle), 1.0, -0.5 * cos(angle));
		vertices[1].normal   := vertices[0].normal;
		vertices[1].texPos.x := 1.0;
		vertices[1].texPos.y := 0.0;
		vertices[2].position := D3DXVector3(vertices[0].position.x, 0.0, vertices[0].position.z);
		vertices[2].normal   := vertices[0].normal;
		vertices[2].texPos.x := 0.0;
		vertices[2].texPos.y := 1.0;
		vertices[3].position := D3DXVector3(vertices[1].position.x, 0.0, vertices[1].position.z);
		vertices[3].normal   := vertices[0].normal;
		vertices[3].texPos.x := 1.0;
		vertices[3].texPos.y := 1.0;

		g_pd3dDevice.SetTexture(0, s_pSprite);
		g_pd3dDevice.SetFVF(D3DFVF_SMVERTEX);
		g_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, vertices, sizeof(TSmVertex));
	end;
	g_pD3DDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, 0);
end;

procedure Render;
var
	s, aux: string;
begin
	// process key events
	if HiByte(GetAsyncKeyState(VK_RIGHT)) > 0 then
		g_3DAttrib.Camera.Angle := g_3DAttrib.Camera.Angle + 0.01;
	if HiByte(GetAsyncKeyState(VK_LEFT)) > 0 then
		g_3DAttrib.Camera.Angle := g_3DAttrib.Camera.Angle - 0.01;
	if HiByte(GetAsyncKeyState(VK_UP)) > 0 then
		g_3DAttrib.Camera.Height := g_3DAttrib.Camera.Height + 0.1;
	if HiByte(GetAsyncKeyState(VK_DOWN)) > 0 then
		g_3DAttrib.Camera.Height := g_3DAttrib.Camera.Height - 0.1;
	if HiByte(GetAsyncKeyState(ord('A'))) > 0 then
		g_3DAttrib.Camera.Radius := g_3DAttrib.Camera.Radius + 0.1;
	if HiByte(GetAsyncKeyState(ord('Z'))) > 0 then
		g_3DAttrib.Camera.Radius := g_3DAttrib.Camera.Radius - 0.1;

	if LoByte(GetAsyncKeyState(VK_SPACE)) > 0 then begin
		if g_3DAttrib.Projection.Mode = pmPerspecive then
			g_3DAttrib.Projection.Mode := pmOrthogonal
		else
			g_3DAttrib.Projection.Mode := pmPerspecive
	end;

	g_3DAttrib.Process();

	// rendering routine
	g_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
					  D3DCOLOR_XRGB(0,0,0), 1.0, 0);

	if SUCCEEDED(g_pd3dDevice.BeginScene) then begin
		SetupMatrices;

		Test();

		str(g_3DAttrib.Camera.Height: 5:2, aux);
		s := 'y = ' + aux;
		str(g_3DAttrib.Camera.Radius: 5:2, aux);
		s := s + ', z = ' + aux;
		g_pFont.DrawTextA(nil, PChar(s), Length(s), nil,  DT_LEFT or DT_TOP, $FFFFFFFF);

		g_pd3dDevice.EndScene;
	end;

	g_pd3dDevice.Present(nil, nil, 0, nil);
end;

function MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  case uMsg of
	WM_DESTROY:
	begin
	  Cleanup;
	  PostQuitMessage(0);
	  Result:= 0;
	  Exit;
	end;
	WM_KEYDOWN:
	if wParam = VK_ESCAPE then begin
	  Cleanup;
	  PostQuitMessage(0);
	  Result:= 0;
	  Exit;
	end;
  end;

  Result:= DefWindowProc(hWnd, uMsg, wParam, lParam);
end;



//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: The application's entry point
//-----------------------------------------------------------------------------
// INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
var
  wc: TWndClassEx = (
    cbSize: SizeOf(TWndClassEx);
	style: CS_CLASSDC;
    lpfnWndProc: @MsgProc;
	cbClsExtra: 0;
	cbWndExtra: 0;
	hInstance: 0; // - filled later
	hIcon: 0;
	hCursor: 0;
	hbrBackground: 0;
	lpszMenuName: nil;
	lpszClassName: 'D3D Tutorial';
	hIconSm: 0);
var
  hWindow: HWND;
  msg: TMsg;
begin
  wc.hInstance:= GetModuleHandle(nil);
  RegisterClassEx(wc);

  // Create the application's window
  hWindow := CreateWindow('D3D Tutorial', 'θ θ ',
						  WS_OVERLAPPEDWINDOW, 100, 100, 640, 480,
						  GetDesktopWindow, 0, wc.hInstance, nil);

  // Initialize Direct3D
  if SUCCEEDED(InitD3D(hWindow)) then
  begin
	begin
	  // Show the window
	  ShowWindow(hWindow, SW_SHOWDEFAULT);
	  UpdateWindow(hWindow);

	  // Enter the message loop
	  FillChar(msg, SizeOf(msg), 0);
	  while (msg.message <> WM_QUIT) do
      begin
        if PeekMessage(msg, 0, 0, 0, PM_REMOVE) then
        begin
          TranslateMessage(msg);
          DispatchMessage(msg);
        end else
          Render;
      end;
    end;
  end;

  UnregisterClass('D3D Tutorial', wc.hInstance);
end.

